@@ -16,7 +16,11 @@
 #include <QGpgME/VerifyDetachedJob>
 #include <QGpgME/DecryptJob>
 #include <QGpgME/DecryptVerifyJob>
+#if QGPGME_SUPPORTS_ARCHIVE_JOBS
+#include <QGpgME/DecryptVerifyArchiveJob>
+#endif
 
+#include <Libkleo/AuditLogEntry>
 #include <Libkleo/Compliance>
 #include <Libkleo/Dn>
 #include <Libkleo/KleoException>
@@ -30,7 +34,6 @@
 #include <utils/input.h>
 #include <utils/output.h>
 #include <utils/kleo_assert.h>
-#include <utils/auditlog.h>
 #include <Libkleo/GnuPG>
 
 #include <KMime/HeaderParsing>
@@ -64,9 +67,9 @@ using namespace KMime::Types;
 namespace
 {
 
-static AuditLog auditLogFromSender(QObject *sender)
+static AuditLogEntry auditLogFromSender(QObject *sender)
 {
-    return AuditLog::fromJob(qobject_cast<const QGpgME::Job *>(sender));
+    return AuditLogEntry::fromJob(qobject_cast<const QGpgME::Job *>(sender));
 }
 
 static bool addrspec_equal(const AddrSpec &lhs, const AddrSpec &rhs, Qt::CaseSensitivity cs)
@@ -653,7 +656,7 @@ public:
             const QString &errString,
             const QString &input,
             const QString &output,
-            const AuditLog &auditLog,
+            const AuditLogEntry &auditLog,
             Task *parentTask,
             const Mailbox &informativeSender,
             DecryptVerifyResult *qq) :
@@ -701,7 +704,7 @@ public:
     QString m_errorString;
     QString m_inputLabel;
     QString m_outputLabel;
-    const AuditLog m_auditLog;
+    const AuditLogEntry m_auditLog;
     QPointer <Task> m_parentTask;
     const Mailbox m_informativeSender;
 };
@@ -711,7 +714,7 @@ DecryptVerifyResult::SenderInfo DecryptVerifyResult::Private::makeSenderInfo() c
     return SenderInfo(m_informativeSender, KeyCache::instance()->findSigners(m_verificationResult));
 }
 
-std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptResult(const DecryptionResult &dr, const QByteArray &plaintext, const AuditLog &auditLog)
+std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptResult(const DecryptionResult &dr, const QByteArray &plaintext, const AuditLogEntry &auditLog)
 {
     return std::shared_ptr<DecryptVerifyResult>(new DecryptVerifyResult(
             Decrypt,
@@ -728,7 +731,7 @@ std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptResul
             informativeSender()));
 }
 
-std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptResult(const GpgME::Error &err, const QString &what, const AuditLog &auditLog)
+std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptResult(const GpgME::Error &err, const QString &what, const AuditLogEntry &auditLog)
 {
     return std::shared_ptr<DecryptVerifyResult>(new DecryptVerifyResult(
             Decrypt,
@@ -745,7 +748,7 @@ std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptResul
             informativeSender()));
 }
 
-std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptVerifyResult(const DecryptionResult &dr, const VerificationResult &vr, const QByteArray &plaintext, const QString &fileName, const AuditLog &auditLog)
+std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptVerifyResult(const DecryptionResult &dr, const VerificationResult &vr, const QByteArray &plaintext, const QString &fileName, const AuditLogEntry &auditLog)
 {
     const auto err = dr.error().code() ? dr.error() : vr.error();
     return std::shared_ptr<DecryptVerifyResult>(new DecryptVerifyResult(
@@ -763,7 +766,7 @@ std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptVerif
             informativeSender()));
 }
 
-std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptVerifyResult(const GpgME::Error &err, const QString &details, const AuditLog &auditLog)
+std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptVerifyResult(const GpgME::Error &err, const QString &details, const AuditLogEntry &auditLog)
 {
     return std::shared_ptr<DecryptVerifyResult>(new DecryptVerifyResult(
             DecryptVerify,
@@ -780,7 +783,7 @@ std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptVerif
             informativeSender()));
 }
 
-std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromVerifyOpaqueResult(const VerificationResult &vr, const QByteArray &plaintext, const AuditLog &auditLog)
+std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromVerifyOpaqueResult(const VerificationResult &vr, const QByteArray &plaintext, const AuditLogEntry &auditLog)
 {
     return std::shared_ptr<DecryptVerifyResult>(new DecryptVerifyResult(
             Verify,
@@ -796,7 +799,7 @@ std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromVerifyOpaque
             this,
             informativeSender()));
 }
-std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromVerifyOpaqueResult(const GpgME::Error &err, const QString &details, const AuditLog &auditLog)
+std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromVerifyOpaqueResult(const GpgME::Error &err, const QString &details, const AuditLogEntry &auditLog)
 {
     return std::shared_ptr<DecryptVerifyResult>(new DecryptVerifyResult(
             Verify,
@@ -813,7 +816,7 @@ std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromVerifyOpaque
             informativeSender()));
 }
 
-std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromVerifyDetachedResult(const VerificationResult &vr, const AuditLog &auditLog)
+std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromVerifyDetachedResult(const VerificationResult &vr, const AuditLogEntry &auditLog)
 {
     return std::shared_ptr<DecryptVerifyResult>(new DecryptVerifyResult(
             Verify,
@@ -829,7 +832,7 @@ std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromVerifyDetach
             this,
             informativeSender()));
 }
-std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromVerifyDetachedResult(const GpgME::Error &err, const QString &details, const AuditLog &auditLog)
+std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromVerifyDetachedResult(const GpgME::Error &err, const QString &details, const AuditLogEntry &auditLog)
 {
     return std::shared_ptr<DecryptVerifyResult>(new DecryptVerifyResult(
             Verify,
@@ -855,7 +858,7 @@ DecryptVerifyResult::DecryptVerifyResult(DecryptVerifyOperation type,
         const QString &errString,
         const QString &inputLabel,
         const QString &outputLabel,
-        const AuditLog &auditLog,
+        const AuditLogEntry &auditLog,
         Task *parentTask,
         const Mailbox &informativeSender)
     : Task::Result(), d(new Private(type, vr, dr, stuff, fileName, error, errString, inputLabel, outputLabel, auditLog, parentTask, informativeSender, this))
@@ -905,7 +908,7 @@ QString DecryptVerifyResult::errorString() const
     return d->m_errorString;
 }
 
-AuditLog DecryptVerifyResult::auditLog() const
+AuditLogEntry DecryptVerifyResult::auditLog() const
 {
     return d->m_auditLog;
 }
@@ -962,33 +965,27 @@ class DecryptVerifyTask::Private
 {
     DecryptVerifyTask *const q;
 public:
-    explicit Private(DecryptVerifyTask *qq) : q(qq), m_backend(nullptr), m_protocol(UnknownProtocol), m_ignoreMDCError(false)  {}
-
-    void slotResult(const DecryptionResult &, const VerificationResult &, const QByteArray &);
-
-    void registerJob(QGpgME::DecryptVerifyJob *job)
+    explicit Private(DecryptVerifyTask *qq)
+        : q{qq}
     {
-        q->connect(job, SIGNAL(result(GpgME::DecryptionResult,GpgME::VerificationResult,QByteArray)),
-                   q, SLOT(slotResult(GpgME::DecryptionResult,GpgME::VerificationResult,QByteArray)));
-        q->connect(job, SIGNAL(progress(QString,int,int)),
-                   q, SLOT(setProgress(QString,int,int)));
     }
 
-    void emitResult(const std::shared_ptr<DecryptVerifyResult> &result);
+    void startDecryptVerifyJob();
+#if QGPGME_SUPPORTS_ARCHIVE_JOBS
+    void startDecryptVerifyArchiveJob();
+#endif
+
+    void slotResult(const DecryptionResult &, const VerificationResult &, const QByteArray & = {});
 
     std::shared_ptr<Input> m_input;
     std::shared_ptr<Output> m_output;
-    const QGpgME::Protocol *m_backend;
-    Protocol m_protocol;
-    bool m_ignoreMDCError;
+    const QGpgME::Protocol *m_backend = nullptr;
+    Protocol m_protocol = UnknownProtocol;
+    bool m_ignoreMDCError = false;
+    bool m_extractArchive = false;
+    QString m_outputDirectory;
 };
 
-void DecryptVerifyTask::Private::emitResult(const std::shared_ptr<DecryptVerifyResult> &result)
-{
-    q->emitResult(result);
-    Q_EMIT q->decryptVerifyResult(result);
-}
-
 void DecryptVerifyTask::Private::slotResult(const DecryptionResult &dr, const VerificationResult &vr, const QByteArray &plainText)
 {
     updateKeys(vr);
@@ -997,34 +994,36 @@ void DecryptVerifyTask::Private::slotResult(const DecryptionResult &dr, const Ve
         ss << dr << '\n' << vr;
         qCDebug(KLEOPATRA_LOG) << ss.str().c_str();
     }
-    const AuditLog auditLog = auditLogFromSender(q->sender());
-    if (dr.error().code() || vr.error().code()) {
-        m_output->cancel();
-    } else {
-        try {
-            kleo_assert(!dr.isNull() || !vr.isNull());
-            m_output->finalize();
-        } catch (const GpgME::Exception &e) {
-            emitResult(q->fromDecryptResult(e.error(), QString::fromLocal8Bit(e.what()), auditLog));
-            return;
-        } catch (const std::exception &e) {
-            emitResult(q->fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), auditLog));
-            return;
-        } catch (...) {
-            emitResult(q->fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), auditLog));
-            return;
+    const AuditLogEntry auditLog = auditLogFromSender(q->sender());
+    if (m_output) {
+        if (dr.error().code() || vr.error().code()) {
+            m_output->cancel();
+        } else {
+            try {
+                kleo_assert(!dr.isNull() || !vr.isNull());
+                m_output->finalize();
+            } catch (const GpgME::Exception &e) {
+                q->emitResult(q->fromDecryptResult(e.error(), QString::fromLocal8Bit(e.what()), auditLog));
+                return;
+            } catch (const std::exception &e) {
+                q->emitResult(q->fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), auditLog));
+                return;
+            } catch (...) {
+                q->emitResult(q->fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), auditLog));
+                return;
+            }
         }
     }
     const int drErr = dr.error().code();
-    const QString errorString = m_output->errorString();
+    const QString errorString = m_output ? m_output->errorString() : QString{};
     if (((drErr == GPG_ERR_EIO || drErr == GPG_ERR_NO_DATA) && !errorString.isEmpty()) ||
-          m_output->failed()) {
-        emitResult(q->fromDecryptResult(drErr ? dr.error() : Error::fromCode(GPG_ERR_EIO),
+          (m_output && m_output->failed())) {
+        q->emitResult(q->fromDecryptResult(drErr ? dr.error() : Error::fromCode(GPG_ERR_EIO),
                     errorString, auditLog));
         return;
     }
 
-    emitResult(q->fromDecryptVerifyResult(dr, vr, plainText, m_output->fileName(), auditLog));
+    q->emitResult(q->fromDecryptVerifyResult(dr, vr, plainText, m_output ? m_output->fileName() : QString{}, auditLog));
 }
 
 DecryptVerifyTask::DecryptVerifyTask(QObject *parent) : AbstractDecryptVerifyTask(parent), d(new Private(this))
@@ -1084,7 +1083,7 @@ QString DecryptVerifyTask::inputLabel() const
 
 QString DecryptVerifyTask::outputLabel() const
 {
-    return d->m_output ? d->m_output->label() : QString();
+    return d->m_output ? d->m_output->label() : d->m_outputDirectory;
 }
 
 Protocol DecryptVerifyTask::protocol() const
@@ -1112,43 +1111,112 @@ void DecryptVerifyTask::setIgnoreMDCError(bool value)
     d->m_ignoreMDCError = value;
 }
 
+void DecryptVerifyTask::setExtractArchive(bool extract)
+{
+    d->m_extractArchive = extract;
+}
+
+void DecryptVerifyTask::setOutputDirectory(const QString &directory)
+{
+    d->m_outputDirectory = directory;
+}
+
+#if QGPGME_SUPPORTS_ARCHIVE_JOBS
+static bool archiveJobsCanBeUsed(GpgME::Protocol protocol)
+{
+    return (protocol == GpgME::OpenPGP) && QGpgME::DecryptVerifyArchiveJob::isSupported();
+}
+#endif
+
 void DecryptVerifyTask::doStart()
 {
     kleo_assert(d->m_backend);
-    try {
-        QGpgME::DecryptVerifyJob *const job = d->m_backend->decryptVerifyJob();
+#if QGPGME_SUPPORTS_ARCHIVE_JOBS
+    if (d->m_extractArchive && archiveJobsCanBeUsed(d->m_protocol)) {
+        d->startDecryptVerifyArchiveJob();
+    } else
+#endif
+    {
+        d->startDecryptVerifyJob();
+    }
+}
 
-        if (d->m_ignoreMDCError) {
-            qCDebug(KLEOPATRA_LOG) << "Modifying job to ignore MDC errors.";
-            auto ctx = QGpgME::Job::context(job);
-            if (!ctx) {
-                qCWarning(KLEOPATRA_LOG) << "Failed to get context for job";
-            } else {
-                const auto err = ctx->setFlag("ignore-mdc-error", "1");
-                if (err) {
-                    qCWarning(KLEOPATRA_LOG) << "Failed to set ignore mdc errors" << err.asString();
-                }
+static void setIgnoreMDCErrorFlag(QGpgME::Job *job, bool ignoreMDCError)
+{
+    if (ignoreMDCError) {
+        qCDebug(KLEOPATRA_LOG) << "Modifying job to ignore MDC errors.";
+        auto ctx = QGpgME::Job::context(job);
+        if (!ctx) {
+            qCWarning(KLEOPATRA_LOG) << "Failed to get context for job";
+        } else {
+            const auto err = ctx->setFlag("ignore-mdc-error", "1");
+            if (err) {
+                qCWarning(KLEOPATRA_LOG) << "Failed to set ignore mdc errors" << err.asString();
             }
         }
-        kleo_assert(job);
-        d->registerJob(job);
-        ensureIOOpen(d->m_input->ioDevice().get(), d->m_output->ioDevice().get());
-        job->start(d->m_input->ioDevice(), d->m_output->ioDevice());
-    } catch (const GpgME::Exception &e) {
-        d->emitResult(fromDecryptVerifyResult(e.error(), QString::fromLocal8Bit(e.what()), AuditLog()));
-    } catch (const std::exception &e) {
-        d->emitResult(fromDecryptVerifyResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), AuditLog()));
-    } catch (...) {
-        d->emitResult(fromDecryptVerifyResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), AuditLog()));
     }
-
 }
 
+void DecryptVerifyTask::Private::startDecryptVerifyJob()
+{
+    try {
+        QGpgME::DecryptVerifyJob *const job = m_backend->decryptVerifyJob();
+        kleo_assert(job);
+        setIgnoreMDCErrorFlag(job, m_ignoreMDCError);
+        QObject::connect(job, &QGpgME::DecryptVerifyJob::result, q, [this](const GpgME::DecryptionResult &decryptResult, const GpgME::VerificationResult &verifyResult, const QByteArray &plainText) {
+            slotResult(decryptResult, verifyResult, plainText);
+        });
+#if QGPGME_JOB_HAS_NEW_PROGRESS_SIGNALS
+        connect(job, &QGpgME::Job::jobProgress, q, &DecryptVerifyTask::setProgress);
+#else
+        QObject::connect(job, &QGpgME::Job::progress, q, [this](const QString &, int processed, int total) {
+            q->setProgress(processed, total);
+        });
+#endif
+        ensureIOOpen(m_input->ioDevice().get(), m_output->ioDevice().get());
+        job->start(m_input->ioDevice(), m_output->ioDevice());
+    } catch (const GpgME::Exception &e) {
+        q->emitResult(q->fromDecryptVerifyResult(e.error(), QString::fromLocal8Bit(e.what()), AuditLogEntry()));
+    } catch (const std::exception &e) {
+        q->emitResult(q->fromDecryptVerifyResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), AuditLogEntry()));
+    } catch (...) {
+        q->emitResult(q->fromDecryptVerifyResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), AuditLogEntry()));
+    }
+}
+
+#if QGPGME_SUPPORTS_ARCHIVE_JOBS
+void DecryptVerifyTask::Private::startDecryptVerifyArchiveJob()
+{
+    auto *const job = m_backend->decryptVerifyArchiveJob();
+    kleo_assert(job);
+    setIgnoreMDCErrorFlag(job, m_ignoreMDCError);
+    job->setOutputDirectory(m_outputDirectory);
+    connect(job, &QGpgME::DecryptVerifyArchiveJob::result, q, [this](const GpgME::DecryptionResult &decryptResult, const GpgME::VerificationResult &verifyResult) {
+        slotResult(decryptResult, verifyResult);
+    });
+#if QGPGME_JOB_HAS_NEW_PROGRESS_SIGNALS
+    connect(job, &QGpgME::Job::jobProgress, q, &DecryptVerifyTask::setProgress);
+#else
+    connect(job, &QGpgME::Job::progress, q, [this](const QString &, int processed, int total) {
+        q->setProgress(processed, total);
+    });
+#endif
+    ensureIOOpen(m_input->ioDevice().get(), nullptr);
+    const auto err = job->start(m_input->ioDevice());
+    if (err) {
+        q->emitResult(q->fromDecryptVerifyResult(err, {}, {}));
+    }
+}
+#endif
+
 class DecryptTask::Private
 {
     DecryptTask *const q;
 public:
-    explicit Private(DecryptTask *qq) : q(qq), m_backend(nullptr), m_protocol(UnknownProtocol)  {}
+    explicit Private(DecryptTask *qq)
+        : q{qq}
+    {
+    }
 
     void slotResult(const DecryptionResult &, const QByteArray &);
 
@@ -1156,24 +1224,21 @@ public:
     {
         q->connect(job, SIGNAL(result(GpgME::DecryptionResult,QByteArray)),
                    q, SLOT(slotResult(GpgME::DecryptionResult,QByteArray)));
-        q->connect(job, SIGNAL(progress(QString,int,int)),
-                   q, SLOT(setProgress(QString,int,int)));
+#if QGPGME_JOB_HAS_NEW_PROGRESS_SIGNALS
+        q->connect(job, &QGpgME::Job::jobProgress, q, &DecryptTask::setProgress);
+#else
+        q->connect(job, &QGpgME::Job::progress, q, [this](const QString &, int processed, int total) {
+            q->setProgress(processed, total);
+        });
+#endif
     }
 
-    void emitResult(const std::shared_ptr<DecryptVerifyResult> &result);
-
     std::shared_ptr<Input> m_input;
     std::shared_ptr<Output> m_output;
-    const QGpgME::Protocol *m_backend;
-    Protocol m_protocol;
+    const QGpgME::Protocol *m_backend = nullptr;
+    Protocol m_protocol = UnknownProtocol;
 };
 
-void DecryptTask::Private::emitResult(const std::shared_ptr<DecryptVerifyResult> &result)
-{
-    q->emitResult(result);
-    Q_EMIT q->decryptVerifyResult(result);
-}
-
 void DecryptTask::Private::slotResult(const DecryptionResult &result, const QByteArray &plainText)
 {
     {
@@ -1181,7 +1246,7 @@ void DecryptTask::Private::slotResult(const DecryptionResult &result, const QByt
         ss << result;
         qCDebug(KLEOPATRA_LOG) << ss.str().c_str();
     }
-    const AuditLog auditLog = auditLogFromSender(q->sender());
+    const AuditLogEntry auditLog = auditLogFromSender(q->sender());
     if (result.error().code()) {
         m_output->cancel();
     } else {
@@ -1189,13 +1254,13 @@ void DecryptTask::Private::slotResult(const DecryptionResult &result, const QByt
             kleo_assert(!result.isNull());
             m_output->finalize();
         } catch (const GpgME::Exception &e) {
-            emitResult(q->fromDecryptResult(e.error(), QString::fromLocal8Bit(e.what()), auditLog));
+            q->emitResult(q->fromDecryptResult(e.error(), QString::fromLocal8Bit(e.what()), auditLog));
             return;
         } catch (const std::exception &e) {
-            emitResult(q->fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), auditLog));
+            q->emitResult(q->fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), auditLog));
             return;
         } catch (...) {
-            emitResult(q->fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), auditLog));
+            q->emitResult(q->fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), auditLog));
             return;
         }
     }
@@ -1204,12 +1269,12 @@ void DecryptTask::Private::slotResult(const DecryptionResult &result, const QByt
     const QString errorString = m_output->errorString();
     if (((drErr == GPG_ERR_EIO || drErr == GPG_ERR_NO_DATA) && !errorString.isEmpty()) ||
           m_output->failed()) {
-        emitResult(q->fromDecryptResult(result.error() ? result.error() : Error::fromCode(GPG_ERR_EIO),
+        q->emitResult(q->fromDecryptResult(result.error() ? result.error() : Error::fromCode(GPG_ERR_EIO),
                     errorString, auditLog));
         return;
     }
 
-    emitResult(q->fromDecryptResult(result, plainText, auditLog));
+    q->emitResult(q->fromDecryptResult(result, plainText, auditLog));
 }
 
 DecryptTask::DecryptTask(QObject *parent) : AbstractDecryptVerifyTask(parent), d(new Private(this))
@@ -1293,11 +1358,11 @@ void DecryptTask::doStart()
         ensureIOOpen(d->m_input->ioDevice().get(), d->m_output->ioDevice().get());
         job->start(d->m_input->ioDevice(), d->m_output->ioDevice());
     } catch (const GpgME::Exception &e) {
-        d->emitResult(fromDecryptResult(e.error(), QString::fromLocal8Bit(e.what()), AuditLog()));
+        emitResult(fromDecryptResult(e.error(), QString::fromLocal8Bit(e.what()), AuditLogEntry()));
     } catch (const std::exception &e) {
-        d->emitResult(fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), AuditLog()));
+        emitResult(fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), AuditLogEntry()));
     } catch (...) {
-        d->emitResult(fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), AuditLog()));
+        emitResult(fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), AuditLogEntry()));
     }
 }
 
@@ -1305,32 +1370,26 @@ class VerifyOpaqueTask::Private
 {
     VerifyOpaqueTask *const q;
 public:
-    explicit Private(VerifyOpaqueTask *qq) : q(qq), m_backend(nullptr), m_protocol(UnknownProtocol)  {}
-
-    void slotResult(const VerificationResult &, const QByteArray &);
-
-    void registerJob(QGpgME::VerifyOpaqueJob *job)
+    explicit Private(VerifyOpaqueTask *qq)
+        : q{qq}
     {
-        q->connect(job, SIGNAL(result(GpgME::VerificationResult,QByteArray)),
-                   q, SLOT(slotResult(GpgME::VerificationResult,QByteArray)));
-        q->connect(job, SIGNAL(progress(QString,int,int)),
-                   q, SLOT(setProgress(QString,int,int)));
     }
 
-    void emitResult(const std::shared_ptr<DecryptVerifyResult> &result);
+    void startVerifyOpaqueJob();
+#if QGPGME_SUPPORTS_ARCHIVE_JOBS
+    void startDecryptVerifyArchiveJob();
+#endif
+
+    void slotResult(const VerificationResult &, const QByteArray & = {});
 
     std::shared_ptr<Input> m_input;
     std::shared_ptr<Output> m_output;
-    const QGpgME::Protocol *m_backend;
-    Protocol m_protocol;
+    const QGpgME::Protocol *m_backend = nullptr;
+    Protocol m_protocol = UnknownProtocol;
+    bool m_extractArchive = false;
+    QString m_outputDirectory;
 };
 
-void VerifyOpaqueTask::Private::emitResult(const std::shared_ptr<DecryptVerifyResult> &result)
-{
-    q->emitResult(result);
-    Q_EMIT q->decryptVerifyResult(result);
-}
-
 void VerifyOpaqueTask::Private::slotResult(const VerificationResult &result, const QByteArray &plainText)
 {
     updateKeys(result);
@@ -1339,35 +1398,37 @@ void VerifyOpaqueTask::Private::slotResult(const VerificationResult &result, con
         ss << result;
         qCDebug(KLEOPATRA_LOG) << ss.str().c_str();
     }
-    const AuditLog auditLog = auditLogFromSender(q->sender());
-    if (result.error().code()) {
-        m_output->cancel();
-    } else {
-        try {
-            kleo_assert(!result.isNull());
-            m_output->finalize();
-        } catch (const GpgME::Exception &e) {
-            emitResult(q->fromDecryptResult(e.error(), QString::fromLocal8Bit(e.what()), auditLog));
-            return;
-        } catch (const std::exception &e) {
-            emitResult(q->fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), auditLog));
-            return;
-        } catch (...) {
-            emitResult(q->fromDecryptResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), auditLog));
-            return;
+    const AuditLogEntry auditLog = auditLogFromSender(q->sender());
+    if (m_output) {
+        if (result.error().code()) {
+            m_output->cancel();
+        } else {
+            try {
+                kleo_assert(!result.isNull());
+                m_output->finalize();
+            } catch (const GpgME::Exception &e) {
+                q->emitResult(q->fromVerifyOpaqueResult(e.error(), QString::fromLocal8Bit(e.what()), auditLog));
+                return;
+            } catch (const std::exception &e) {
+                q->emitResult(q->fromVerifyOpaqueResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), auditLog));
+                return;
+            } catch (...) {
+                q->emitResult(q->fromVerifyOpaqueResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), auditLog));
+                return;
+            }
         }
     }
 
     const int drErr = result.error().code();
-    const QString errorString = m_output->errorString();
+    const QString errorString = m_output ? m_output->errorString() : QString{};
     if (((drErr == GPG_ERR_EIO || drErr == GPG_ERR_NO_DATA) && !errorString.isEmpty()) ||
-          m_output->failed()) {
-        emitResult(q->fromDecryptResult(result.error() ? result.error() : Error::fromCode(GPG_ERR_EIO),
+          (m_output && m_output->failed())) {
+        q->emitResult(q->fromVerifyOpaqueResult(result.error() ? result.error() : Error::fromCode(GPG_ERR_EIO),
                     errorString, auditLog));
         return;
     }
 
-    emitResult(q->fromVerifyOpaqueResult(result, plainText, auditLog));
+    q->emitResult(q->fromVerifyOpaqueResult(result, plainText, auditLog));
 }
 
 VerifyOpaqueTask::VerifyOpaqueTask(QObject *parent) : AbstractDecryptVerifyTask(parent), d(new Private(this))
@@ -1427,7 +1488,7 @@ QString VerifyOpaqueTask::inputLabel() const
 
 QString VerifyOpaqueTask::outputLabel() const
 {
-    return d->m_output ? d->m_output->label() : QString();
+    return d->m_output ? d->m_output->label() : d->m_outputDirectory;
 }
 
 Protocol VerifyOpaqueTask::protocol() const
@@ -1435,6 +1496,16 @@ Protocol VerifyOpaqueTask::protocol() const
     return d->m_protocol;
 }
 
+void VerifyOpaqueTask::setExtractArchive(bool extract)
+{
+    d->m_extractArchive = extract;
+}
+
+void VerifyOpaqueTask::setOutputDirectory(const QString &directory)
+{
+    d->m_outputDirectory = directory;
+}
+
 void VerifyOpaqueTask::cancel()
 {
 
@@ -1443,27 +1514,68 @@ void VerifyOpaqueTask::cancel()
 void VerifyOpaqueTask::doStart()
 {
     kleo_assert(d->m_backend);
-
-    try {
-        QGpgME::VerifyOpaqueJob *const job = d->m_backend->verifyOpaqueJob();
-        kleo_assert(job);
-        d->registerJob(job);
-        ensureIOOpen(d->m_input->ioDevice().get(), d->m_output ? d->m_output->ioDevice().get() : nullptr);
-        job->start(d->m_input->ioDevice(), d->m_output ? d->m_output->ioDevice() : std::shared_ptr<QIODevice>());
-    } catch (const GpgME::Exception &e) {
-        d->emitResult(fromVerifyOpaqueResult(e.error(), QString::fromLocal8Bit(e.what()), AuditLog()));
-    } catch (const std::exception &e) {
-        d->emitResult(fromVerifyOpaqueResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), AuditLog()));
-    } catch (...) {
-        d->emitResult(fromVerifyOpaqueResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), AuditLog()));
+#if QGPGME_SUPPORTS_ARCHIVE_JOBS
+    if (d->m_extractArchive && archiveJobsCanBeUsed(d->m_protocol)) {
+        d->startDecryptVerifyArchiveJob();
+    } else
+#endif
+    {
+        d->startVerifyOpaqueJob();
     }
 }
 
+void VerifyOpaqueTask::Private::startVerifyOpaqueJob()
+{
+    try {
+        QGpgME::VerifyOpaqueJob *const job = m_backend->verifyOpaqueJob();
+        kleo_assert(job);
+        connect(job, &QGpgME::VerifyOpaqueJob::result, q, [this](const GpgME::VerificationResult &result, const QByteArray &plainText) {
+            slotResult(result, plainText);
+        });
+#if QGPGME_JOB_HAS_NEW_PROGRESS_SIGNALS
+        connect(job, &QGpgME::Job::jobProgress, q, &VerifyOpaqueTask::setProgress);
+#else
+        connect(job, &QGpgME::Job::progress, q, [this](const QString &, int processed, int total) {
+            q->setProgress(processed, total);
+        });
+#endif
+        ensureIOOpen(m_input->ioDevice().get(), m_output ? m_output->ioDevice().get() : nullptr);
+        job->start(m_input->ioDevice(), m_output ? m_output->ioDevice() : std::shared_ptr<QIODevice>());
+    } catch (const GpgME::Exception &e) {
+        q->emitResult(q->fromVerifyOpaqueResult(e.error(), QString::fromLocal8Bit(e.what()), AuditLogEntry()));
+    } catch (const std::exception &e) {
+        q->emitResult(q->fromVerifyOpaqueResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), AuditLogEntry()));
+    } catch (...) {
+        q->emitResult(q->fromVerifyOpaqueResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), AuditLogEntry()));
+    }
+}
+
+#if QGPGME_SUPPORTS_ARCHIVE_JOBS
+void VerifyOpaqueTask::Private::startDecryptVerifyArchiveJob()
+{
+    auto *const job = m_backend->decryptVerifyArchiveJob();
+    kleo_assert(job);
+    job->setOutputDirectory(m_outputDirectory);
+    connect(job, &QGpgME::DecryptVerifyArchiveJob::result, q, [this](const DecryptionResult &, const VerificationResult &verifyResult) {
+        slotResult(verifyResult);
+    });
+    connect(job, &QGpgME::DecryptVerifyArchiveJob::dataProgress, q, &VerifyOpaqueTask::setProgress);
+    ensureIOOpen(m_input->ioDevice().get(), nullptr);
+    const auto err = job->start(m_input->ioDevice());
+    if (err) {
+        q->emitResult(q->fromVerifyOpaqueResult(err, {}, {}));
+    }
+}
+#endif
+
 class VerifyDetachedTask::Private
 {
     VerifyDetachedTask *const q;
 public:
-    explicit Private(VerifyDetachedTask *qq) : q(qq), m_backend(nullptr), m_protocol(UnknownProtocol) {}
+    explicit Private(VerifyDetachedTask *qq)
+        : q{qq}
+    {
+    }
 
     void slotResult(const VerificationResult &);
 
@@ -1471,23 +1583,20 @@ public:
     {
         q->connect(job, SIGNAL(result(GpgME::VerificationResult)),
                    q, SLOT(slotResult(GpgME::VerificationResult)));
-        q->connect(job, SIGNAL(progress(QString,int,int)),
-                   q, SLOT(setProgress(QString,int,int)));
+#if QGPGME_JOB_HAS_NEW_PROGRESS_SIGNALS
+        q->connect(job, &QGpgME::Job::jobProgress, q, &VerifyDetachedTask::setProgress);
+#else
+        q->connect(job, &QGpgME::Job::progress, q, [this](const QString &, int processed, int total) {
+            q->setProgress(processed, total);
+        });
+#endif
     }
 
-    void emitResult(const std::shared_ptr<DecryptVerifyResult> &result);
-
     std::shared_ptr<Input> m_input, m_signedData;
-    const QGpgME::Protocol *m_backend;
-    Protocol m_protocol;
+    const QGpgME::Protocol *m_backend = nullptr;
+    Protocol m_protocol = UnknownProtocol;
 };
 
-void VerifyDetachedTask::Private::emitResult(const std::shared_ptr<DecryptVerifyResult> &result)
-{
-    q->emitResult(result);
-    Q_EMIT q->decryptVerifyResult(result);
-}
-
 void VerifyDetachedTask::Private::slotResult(const VerificationResult &result)
 {
     updateKeys(result);
@@ -1496,16 +1605,16 @@ void VerifyDetachedTask::Private::slotResult(const VerificationResult &result)
         ss << result;
         qCDebug(KLEOPATRA_LOG) << ss.str().c_str();
     }
-    const AuditLog auditLog = auditLogFromSender(q->sender());
+    const AuditLogEntry auditLog = auditLogFromSender(q->sender());
     try {
         kleo_assert(!result.isNull());
-        emitResult(q->fromVerifyDetachedResult(result, auditLog));
+        q->emitResult(q->fromVerifyDetachedResult(result, auditLog));
     } catch (const GpgME::Exception &e) {
-        emitResult(q->fromVerifyDetachedResult(e.error(), QString::fromLocal8Bit(e.what()), auditLog));
+        q->emitResult(q->fromVerifyDetachedResult(e.error(), QString::fromLocal8Bit(e.what()), auditLog));
     } catch (const std::exception &e) {
-        emitResult(q->fromVerifyDetachedResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), auditLog));
+        q->emitResult(q->fromVerifyDetachedResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), auditLog));
     } catch (...) {
-        emitResult(q->fromVerifyDetachedResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), auditLog));
+        q->emitResult(q->fromVerifyDetachedResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), auditLog));
     }
 }
 
@@ -1606,11 +1715,11 @@ void VerifyDetachedTask::doStart()
         ensureIOOpen(d->m_signedData->ioDevice().get(), nullptr);
         job->start(d->m_input->ioDevice(), d->m_signedData->ioDevice());
     } catch (const GpgME::Exception &e) {
-        d->emitResult(fromVerifyDetachedResult(e.error(), QString::fromLocal8Bit(e.what()), AuditLog()));
+        emitResult(fromVerifyDetachedResult(e.error(), QString::fromLocal8Bit(e.what()), AuditLogEntry()));
     } catch (const std::exception &e) {
-        d->emitResult(fromVerifyDetachedResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), AuditLog()));
+        emitResult(fromVerifyDetachedResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught exception: %1", QString::fromLocal8Bit(e.what())), AuditLogEntry()));
     } catch (...) {
-        d->emitResult(fromVerifyDetachedResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), AuditLog()));
+        emitResult(fromVerifyDetachedResult(Error::fromCode(GPG_ERR_INTERNAL), i18n("Caught unknown exception"), AuditLogEntry()));
     }
 }
 
