@@ -36,7 +36,7 @@ namespace Kleo
 {
 class Input;
 class Output;
-class AuditLog;
+class AuditLogEntry;
 }
 
 namespace Kleo
@@ -60,18 +60,15 @@ public:
     virtual QString inputLabel() const = 0;
     virtual QString outputLabel() const = 0;
 
-Q_SIGNALS:
-    void decryptVerifyResult(const std::shared_ptr<const Kleo::Crypto::DecryptVerifyResult> &);
-
 protected:
-    std::shared_ptr<DecryptVerifyResult> fromDecryptResult(const GpgME::DecryptionResult &dr, const QByteArray &plaintext, const AuditLog &auditLog);
-    std::shared_ptr<DecryptVerifyResult> fromDecryptResult(const GpgME::Error &err, const QString &details, const AuditLog &auditLog);
-    std::shared_ptr<DecryptVerifyResult> fromDecryptVerifyResult(const GpgME::DecryptionResult &dr, const GpgME::VerificationResult &vr, const QByteArray &plaintext, const QString &fileName, const AuditLog &auditLog);
-    std::shared_ptr<DecryptVerifyResult> fromDecryptVerifyResult(const GpgME::Error &err, const QString &what, const AuditLog &auditLog);
-    std::shared_ptr<DecryptVerifyResult> fromVerifyOpaqueResult(const GpgME::VerificationResult &vr, const QByteArray &plaintext, const AuditLog &auditLog);
-    std::shared_ptr<DecryptVerifyResult> fromVerifyOpaqueResult(const GpgME::Error &err, const QString &details, const AuditLog &auditLog);
-    std::shared_ptr<DecryptVerifyResult> fromVerifyDetachedResult(const GpgME::VerificationResult &vr, const AuditLog &auditLog);
-    std::shared_ptr<DecryptVerifyResult> fromVerifyDetachedResult(const GpgME::Error &err, const QString &details, const AuditLog &auditLog);
+    std::shared_ptr<DecryptVerifyResult> fromDecryptResult(const GpgME::DecryptionResult &dr, const QByteArray &plaintext, const AuditLogEntry &auditLog);
+    std::shared_ptr<DecryptVerifyResult> fromDecryptResult(const GpgME::Error &err, const QString &details, const AuditLogEntry &auditLog);
+    std::shared_ptr<DecryptVerifyResult> fromDecryptVerifyResult(const GpgME::DecryptionResult &dr, const GpgME::VerificationResult &vr, const QByteArray &plaintext, const QString &fileName, const AuditLogEntry &auditLog);
+    std::shared_ptr<DecryptVerifyResult> fromDecryptVerifyResult(const GpgME::Error &err, const QString &what, const AuditLogEntry &auditLog);
+    std::shared_ptr<DecryptVerifyResult> fromVerifyOpaqueResult(const GpgME::VerificationResult &vr, const QByteArray &plaintext, const AuditLogEntry &auditLog);
+    std::shared_ptr<DecryptVerifyResult> fromVerifyOpaqueResult(const GpgME::Error &err, const QString &details, const AuditLogEntry &auditLog);
+    std::shared_ptr<DecryptVerifyResult> fromVerifyDetachedResult(const GpgME::VerificationResult &vr, const AuditLogEntry &auditLog);
+    std::shared_ptr<DecryptVerifyResult> fromVerifyDetachedResult(const GpgME::Error &err, const QString &details, const AuditLogEntry &auditLog);
 
 private:
     class Private;
@@ -160,6 +157,9 @@ public:
     QString label() const override;
     GpgME::Protocol protocol() const override;
 
+    void setExtractArchive(bool extract);
+    void setOutputDirectory(const QString &directory);
+
     QString inputLabel() const override;
     QString outputLabel() const override;
 
@@ -184,7 +184,6 @@ public:
     ~DecryptVerifyTask() override;
 
     void setInput(const std::shared_ptr<Input> &input);
-    void setSignedData(const std::shared_ptr<Input> &signedData);
     void setOutput(const std::shared_ptr<Output> &output);
 
     void setProtocol(GpgME::Protocol prot);
@@ -195,6 +194,8 @@ public:
     GpgME::Protocol protocol() const override;
 
     void setIgnoreMDCError(bool value);
+    void setExtractArchive(bool extract);
+    void setOutputDirectory(const QString &directory);
 
     QString inputLabel() const override;
     QString outputLabel() const override;
@@ -223,7 +224,7 @@ public:
     GpgME::Error error() const override;
     QString errorString() const override;
     VisualCode code() const override;
-    AuditLog auditLog() const override;
+    AuditLogEntry auditLog() const override;
     QPointer<Task> parentTask() const override;
 
     GpgME::VerificationResult verificationResult() const;
@@ -247,7 +248,7 @@ private:
                         const QString &errString,
                         const QString &inputLabel,
                         const QString &outputLabel,
-                        const AuditLog &auditLog,
+                        const AuditLogEntry &auditLog,
                         Task *parentTask,
                         const KMime::Types::Mailbox &informativeSender);
 
