@@ -66,7 +66,6 @@ DecryptVerifyFilesDialog::DecryptVerifyFilesDialog(const std::shared_ptr<TaskCol
     m_resultList->setTaskCollection(coll);
     connect(m_tasks.get(), &TaskCollection::progress, this, &DecryptVerifyFilesDialog::progress);
     connect(m_tasks.get(), &TaskCollection::done, this, &DecryptVerifyFilesDialog::allDone);
-    connect(m_tasks.get(), &TaskCollection::result, this, &DecryptVerifyFilesDialog::result);
     connect(m_tasks.get(), &TaskCollection::started, this, &DecryptVerifyFilesDialog::started);
 
     connect(m_buttonBox, &QDialogButtonBox::rejected, this, &QDialog::reject);
@@ -162,9 +161,8 @@ QLabel *DecryptVerifyFilesDialog::labelForTag(const QString &tag)
     return label;
 }
 
-void DecryptVerifyFilesDialog::progress(const QString &msg, int progress, int total)
+void DecryptVerifyFilesDialog::progress(int progress, int total)
 {
-    Q_UNUSED(msg)
     Q_ASSERT(progress >= 0);
     Q_ASSERT(total >= 0);
     m_progressBar->setRange(0, total);
