@@ -27,6 +27,7 @@
 
 #include "dialogs/gencardkeydialog.h"
 
+#include <Libkleo/Compliance>
 #include <Libkleo/GnuPG>
 
 #include <QProgressDialog>
@@ -59,10 +60,12 @@ using namespace Kleo;
 using namespace Kleo::Commands;
 using namespace Kleo::SmartCard;
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 5, 0)
 static QDebug operator<<(QDebug s, const std::string &string)
 {
     return s << QString::fromStdString(string);
 }
+#endif
 
 namespace {
 class GenKeyThread: public QThread
@@ -91,7 +94,7 @@ class GenKeyThread: public QThread
             // minus 1 of GpgGenCardKeyInteractor::Curve
             static const std::vector<std::string> curves = {
                 "curve25519",
-#ifdef GPGMEPP_SUPPORTS_SET_CURVE
+#if GPGMEPP_SUPPORTS_SET_CURVE
                 "curve448",
                 "nistp256",
                 "nistp384",
@@ -111,7 +114,7 @@ class GenKeyThread: public QThread
                 ei->setAlgo(GpgME::GpgGenCardKeyInteractor::ECC);
                 const auto curveIt = std::find(curves.cbegin(), curves.cend(), mParams.algorithm);
                 if (curveIt != curves.end()) {
-#ifdef GPGMEPP_SUPPORTS_SET_CURVE
+#if GPGMEPP_SUPPORTS_SET_CURVE
                     ei->setCurve(static_cast<GpgME::GpgGenCardKeyInteractor::Curve>(curveIt - curves.cbegin() + 1));
 #endif
                 } else {
@@ -304,7 +307,10 @@ void PGPCardWidget::setCard(const OpenPGPCard *card)
         && card->keyFingerprint(OpenPGPCard::pgpAuthKeyRef()).empty();
 
     if (mKeyForCardKeysButton) {
-        mKeyForCardKeysButton->setEnabled(card->hasSigningKey() && card->hasEncryptionKey());
+        mKeyForCardKeysButton->setEnabled(card->hasSigningKey()
+                                          && card->hasEncryptionKey()
+                                          && DeVSCompliance::algorithmIsCompliant(card->keyInfo(card->signingKeyRef()).algorithm)
+                                          && DeVSCompliance::algorithmIsCompliant(card->keyInfo(card->encryptionKeyRef()).algorithm));
     }
 }
 
@@ -404,11 +410,10 @@ void PGPCardWidget::genkeyRequested()
     }
 
     auto dlg = new GenCardKeyDialog(GenCardKeyDialog::AllKeyAttributes, this);
-#ifdef GPGMEPP_SUPPORTS_SET_CURVE
+#if GPGMEPP_SUPPORTS_SET_CURVE
     dlg->setSupportedAlgorithms(pgpCard->supportedAlgorithms(), pgpCard->defaultAlgorithm());
 #else
     std::vector<AlgorithmInfo> algos = {
-        { "curve25519", i18nc("@info", "ECC (Curve25519)") },
         { "rsa2048", i18nc("@info", "RSA 2048") },
         { "rsa3072", i18nc("@info", "RSA 3072") },
     };
@@ -416,6 +421,12 @@ void PGPCardWidget::genkeyRequested()
     if (mIs21) {
         algos.push_back({"rsa4096", i18nc("@info", "RSA 4096")});
     }
+    const auto supportedAlgos = pgpCard->supportedAlgorithms();
+    if (std::any_of(supportedAlgos.begin(), supportedAlgos.end(), [](const auto &algo) {
+        return algo.id == "curve25519";
+    })) {
+        algos.push_back({"curve25519", i18nc("@info", "ECC (Curve25519)")});
+    }
     dlg->setSupportedAlgorithms(algos, "rsa2048");
 #endif
     connect(dlg, &QDialog::accepted, this, [this, dlg] () {
